#!/usr/bin/perl

use strict;
use warnings;

use DBI;
use English qw( -no_match_vars );
use URI;
use Getopt::Long;

# script options
my $group_cold_sites   = 0;     # group sites that have only $group_threshold requests in a "others" entry ?
my $group_threshold    = 1;     # consider "cold" sites that have these many requests
my $hide_others        = 0;     # should we print the 'others' entry ?

my $result = GetOptions(
    'group-cold-sites'   => \$group_cold_sites,
    'group-threshold=i'  => \$group_threshold,
    'hide-others'        => \$hide_others,
);

if (!$result) {
    exit -1;
}


sub log_info {
    my $msg = shift;
    print STDERR "$msg\n";
}


my $log_file = shift;

my ( $host, $database, $table, $user, $pass ) = $log_file =~ / \/(.*?)\/(.*?)\/(.*?)\/(.*?)\/(.*?) \z /xms;

if ( !$host ) {
    $host = 'localhost';
    log_info("Database host not specified. Using $host.");
}

if ( !$database ) {
    $database = 'squid_log';
    log_info("Database name not specified. Using $database.");
}

if ( !$table ) {
    $table = 'access_log';
    log_info("Table parameter not specified. Using $table.");
}

if ( !$user ) {
    $user = 'squid';
    log_info("User parameter not specified. Using $user.");
}

if ( !$pass ) {
    log_info('No password specified. Connecting with NO password.');
}


# fields that we should have in the table
my @fields = qw(
    request_url
);

my $dsn;
my $dbh;
my $sth;

# perform db connection
$dsn = "DBI:mysql:database=$database";
eval {
    warn "Connecting... dsn='$dsn', username='$user', password='...'";
    $dbh = DBI->connect($dsn, $user, $pass, { AutoCommit => 1, RaiseError => 1, PrintError => 1 });
};
if ($EVAL_ERROR) {
    die "Cannot connect to database: $DBI::errstr";
}


# prepare and execute select statement
eval {
    my $q = 'SELECT ' . join(',',@fields) . " FROM $table";
    $sth = $dbh->prepare($q);
    $sth->execute();
};
if ($EVAL_ERROR) {
    die "Error while preparing or executing sql statement: $EVAL_ERROR";
}


my %hosts;

# process rows
while (my $row = $sth->fetchrow_hashref) {
    my $url = $row->{request_url};
    my $uri_obj = URI->new($url);
    eval {
        my $host = $uri_obj->host;
        $hosts{$host}++;
    };
    if ($EVAL_ERROR) {
        $hosts{$url}++;
    }
}

$dbh->disconnect();


# grouping of cold sites requested ?
if ($group_cold_sites) {
    if ($group_threshold <= 0) {
        log_info('group-threshold cannot be 0 or less. Assuming 1.');
        $group_threshold = 1;
    }
    
    log_info("Grouping sites with $group_threshold or less requests as 'others'.");

    my $others = 0;
    foreach my $k (keys %hosts) {
        if ($hosts{$k} <= $group_threshold) {
            $others += $hosts{$k};
            delete $hosts{$k};
        }
    }
    
    if (($others > 0) and (!$hide_others)) {
        $hosts{'others'} = -$others;        # NOTE: this ensures that the 'others' entry is always displayed last
    }
}

# print hosts sorted by number of requests, hottest sites first
foreach my $k (sort { $hosts{$b} <=> $hosts{$a} } keys %hosts) {
    print "\"$k\", " . ($hosts{$k} >= 0 ? $hosts{$k} : -$hosts{$k}) . "\n";
}

__END__

=head1 NAME

list_hostnames.pl - List hostsnames from a mysql squid log

=head1 SYNOPSIS

  list_hostnames.pl > hosts.csv

  list_hostnames.pl /////password > hosts.csv

  list_hostnames.pl /mysql_host/mysql_database/table_name/username/password > hosts.csv

=head1 DESCRIPTION

This script prints the hostname part of the C<request_url> field of each entry in a mysql squid access log, along with the number of requests to that hostname. The list is sorted by number of request in descending order, so the hottest sites come first.

The output format is CSV (Comma Separated Values). The first field is the hostname, surrounded by double quotes, the second field is the number of requests, unquoted.

=head1 OPTIONS

=over 4

=item --group-cold-sites

Normally all hosts are printed, including those with only a few (or even only one) requests. If this option is specified, sites with only one request are not printed, but summed up in a 'others' entry, always printed last.

If you want to group sites with more than one request, use the L<--group-threshold option>, described below.

=item --group-threshold

Number of request that a site must have to be included in the "others" entry. If not specified, defaults to 1.

Only useful if L<--group-cold-sites> is specified.

=item --hide-others

If specified, the "others" entry is not displayed.

Only useful if L<--group-cold-sites> is specified.

=back

=head1 SEE ALSO

L<logfile-daemon_mysql.pl>

L<http://www.squid.org>

=head1 AUTHOR

Marcello Romani, marcello.romani@libero.it

=head1 COPYRIGHT AND LICENSE

Copyright (C) 2008 by Marcello Romani

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.8 or,
at your option, any later version of Perl 5 you may have available.

=cut
