#!/usr/bin/perl

use strict;
use warnings;

use DBI;
use English qw( -no_match_vars );

my $dsn;
my $dbh;
my $sth;

my $first_req_date;
my $last_req_date;

my $q;              # temp buffer for sql queries
my $count_sth;      # to hold the prepared statements
my $next_date_sth;

my $has_date_day_column;    # true (1) if access log table has date_day column, false (0) otherwise


sub log_info {
    my $msg = shift;
    print STDERR "$msg\n";
    return;
}

#
# command line processing
#
my $log_file = shift;
my ( $host, $database, $table, $user, $pass ) = $log_file =~ / \/(.*?)\/(.*?)\/(.*?)\/(.*?)\/(.*?) \z /xms;

if ( !$host ) {
    $host = 'localhost';
    log_info("Database host not specified. Using $host.");
}

if ( !$database ) {
    $database = 'squid_log';
    log_info("Database name not specified. Using $database.");
}

if ( !$table ) {
    $table = 'access_log';
    log_info("Table parameter not specified. Using $table.");
}

if ( !$user ) {
    $user = 'squid';
    log_info("User parameter not specified. Using $user.");
}

if ( !$pass ) {
    log_info('No password specified. Connecting with NO password.');
}


#
# database connection
#
$dsn = "DBI:mysql:database=$database";
eval {
    warn "Connecting... dsn='$dsn', username='$user', password='...'";
    $dbh = DBI->connect($dsn, $user, $pass, { AutoCommit => 1, RaiseError => 1, PrintError => 1 });
};
if ($EVAL_ERROR) {
    die "Cannot connect to database: $DBI::errstr";
}


#
# let's see if we have the date_day column
#
eval {
    $q = "SELECT date_day FROM $table LIMIT 1";
    $sth = $dbh->prepare($q);
    $sth->execute();
    $has_date_day_column = 1;       # ok, we have the column
};
if ($EVAL_ERROR) {
    if ($EVAL_ERROR =~ / Unknown \s+ column /xims) {
        $has_date_day_column = 0;   # selecting the column generated an error, so we don't have it
    }
    else {                          # there was a database error which apparently in not related to date_day absence
        die "Error while trying to detect date_day column presence: $DBI::errstr";
    }
}
warn 'Do we have date_day column ? ' . ($has_date_day_column ? 'Yes' : 'No, reverting to slow FROM_UNIXTIME method.');



#
# obtain first and last day
#
if ($has_date_day_column) {
    $q = 'SELECT MIN(date_day) AS min_date, MAX(date_day) AS max_date FROM access_log';
}
else {
    $q = 'SELECT MIN(DATE(FROM_UNIXTIME(time_since_epoch))) AS min_date, MAX(DATE(FROM_UNIXTIME(time_since_epoch))) AS max_date FROM access_log';
}

$sth = $dbh->prepare($q);
$sth->execute();
($first_req_date, $last_req_date) = $sth->fetchrow_array;
$sth->finish;

#print "First date: $first_req_date\n";
#print "Last date: $last_req_date\n";

#
# prepare queries
#
if ($has_date_day_column) {
    $count_sth     = $dbh->prepare("SELECT COUNT(*) FROM $table WHERE date_day = ?");
}
else {
    $count_sth     = $dbh->prepare("SELECT COUNT(*) FROM $table WHERE DATE(FROM_UNIXTIME(time_since_epoch)) = ?");
}
$next_date_sth = $dbh->prepare("SELECT ADDDATE(DATE( ? ), INTERVAL 1 DAY)");

#
# count requests by date
#
my $d = $first_req_date;
while ($d le $last_req_date) {
    $count_sth->execute($d);
    my ($n) = $count_sth->fetchrow_array;
    print "\"$d\",$n\n";

    $next_date_sth->execute($d);
    ($d) = $next_date_sth->fetchrow_array;
}

$count_sth->finish;
$next_date_sth->finish;

$dbh->disconnect();
